"""
Type annotations for iot-data service client paginators.

[Documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/paginators/)

Copyright 2025 Vlad Emelianov

Usage::

    ```python
    from boto3.session import Session

    from types_boto3_iot_data.client import IoTDataPlaneClient
    from types_boto3_iot_data.paginator import (
        ListRetainedMessagesPaginator,
    )

    session = Session()
    client: IoTDataPlaneClient = session.client("iot-data")

    list_retained_messages_paginator: ListRetainedMessagesPaginator = client.get_paginator("list_retained_messages")
    ```
"""

from __future__ import annotations

import sys
from typing import TYPE_CHECKING

from botocore.paginate import PageIterator, Paginator

from .type_defs import (
    ListRetainedMessagesRequestPaginateTypeDef,
    ListRetainedMessagesResponseTypeDef,
)

if sys.version_info >= (3, 12):
    from typing import Unpack
else:
    from typing_extensions import Unpack


__all__ = ("ListRetainedMessagesPaginator",)


if TYPE_CHECKING:
    _ListRetainedMessagesPaginatorBase = Paginator[ListRetainedMessagesResponseTypeDef]
else:
    _ListRetainedMessagesPaginatorBase = Paginator  # type: ignore[assignment]


class ListRetainedMessagesPaginator(_ListRetainedMessagesPaginatorBase):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/paginator/ListRetainedMessages.html#IoTDataPlane.Paginator.ListRetainedMessages)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/paginators/#listretainedmessagespaginator)
    """

    def paginate(  # type: ignore[override]
        self, **kwargs: Unpack[ListRetainedMessagesRequestPaginateTypeDef]
    ) -> PageIterator[ListRetainedMessagesResponseTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/paginator/ListRetainedMessages.html#IoTDataPlane.Paginator.ListRetainedMessages.paginate)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/paginators/#listretainedmessagespaginator)
        """



"""
Main interface for iot-data service.

[Documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/)

Copyright 2025 Vlad Emelianov

Usage::

    ```python
    from boto3.session import Session
    from types_boto3_iot_data import (
        Client,
        IoTDataPlaneClient,
        ListRetainedMessagesPaginator,
    )

    session = Session()
    client: IoTDataPlaneClient = session.client("iot-data")

    list_retained_messages_paginator: ListRetainedMessagesPaginator = client.get_paginator("list_retained_messages")
    ```
"""

from .client import IoTDataPlaneClient
from .paginator import ListRetainedMessagesPaginator

Client = IoTDataPlaneClient


__all__ = ("Client", "IoTDataPlaneClient", "ListRetainedMessagesPaginator")



"""
Type annotations for iot-data service Client.

[Documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/)

Copyright 2025 Vlad Emelianov

Usage::

    ```python
    from boto3.session import Session
    from types_boto3_iot_data.client import IoTDataPlaneClient

    session = Session()
    client: IoTDataPlaneClient = session.client("iot-data")
    ```
"""

from __future__ import annotations

import sys
from typing import Any

from botocore.client import BaseClient, ClientMeta
from botocore.errorfactory import BaseClientExceptions
from botocore.exceptions import ClientError as BotocoreClientError

from .paginator import ListRetainedMessagesPaginator
from .type_defs import (
    DeleteThingShadowRequestTypeDef,
    DeleteThingShadowResponseTypeDef,
    EmptyResponseMetadataTypeDef,
    GetRetainedMessageRequestTypeDef,
    GetRetainedMessageResponseTypeDef,
    GetThingShadowRequestTypeDef,
    GetThingShadowResponseTypeDef,
    ListNamedShadowsForThingRequestTypeDef,
    ListNamedShadowsForThingResponseTypeDef,
    ListRetainedMessagesRequestTypeDef,
    ListRetainedMessagesResponseTypeDef,
    PublishRequestTypeDef,
    UpdateThingShadowRequestTypeDef,
    UpdateThingShadowResponseTypeDef,
)

if sys.version_info >= (3, 9):
    from builtins import type as Type
    from collections.abc import Mapping
else:
    from typing import Mapping, Type
if sys.version_info >= (3, 12):
    from typing import Literal, Unpack
else:
    from typing_extensions import Literal, Unpack


__all__ = ("IoTDataPlaneClient",)


class Exceptions(BaseClientExceptions):
    ClientError: Type[BotocoreClientError]
    ConflictException: Type[BotocoreClientError]
    InternalFailureException: Type[BotocoreClientError]
    InvalidRequestException: Type[BotocoreClientError]
    MethodNotAllowedException: Type[BotocoreClientError]
    RequestEntityTooLargeException: Type[BotocoreClientError]
    ResourceNotFoundException: Type[BotocoreClientError]
    ServiceUnavailableException: Type[BotocoreClientError]
    ThrottlingException: Type[BotocoreClientError]
    UnauthorizedException: Type[BotocoreClientError]
    UnsupportedDocumentEncodingException: Type[BotocoreClientError]


class IoTDataPlaneClient(BaseClient):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data.html#IoTDataPlane.Client)
    [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/)
    """

    meta: ClientMeta

    @property
    def exceptions(self) -> Exceptions:
        """
        IoTDataPlaneClient exceptions.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data.html#IoTDataPlane.Client)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#exceptions)
        """

    def can_paginate(self, operation_name: str) -> bool:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/client/can_paginate.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#can_paginate)
        """

    def generate_presigned_url(
        self,
        ClientMethod: str,
        Params: Mapping[str, Any] = ...,
        ExpiresIn: int = 3600,
        HttpMethod: str = ...,
    ) -> str:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/client/generate_presigned_url.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#generate_presigned_url)
        """

    def delete_thing_shadow(
        self, **kwargs: Unpack[DeleteThingShadowRequestTypeDef]
    ) -> DeleteThingShadowResponseTypeDef:
        """
        Deletes the shadow for the specified thing.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/client/delete_thing_shadow.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#delete_thing_shadow)
        """

    def get_retained_message(
        self, **kwargs: Unpack[GetRetainedMessageRequestTypeDef]
    ) -> GetRetainedMessageResponseTypeDef:
        """
        Gets the details of a single retained message for the specified topic.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/client/get_retained_message.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#get_retained_message)
        """

    def get_thing_shadow(
        self, **kwargs: Unpack[GetThingShadowRequestTypeDef]
    ) -> GetThingShadowResponseTypeDef:
        """
        Gets the shadow for the specified thing.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/client/get_thing_shadow.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#get_thing_shadow)
        """

    def list_named_shadows_for_thing(
        self, **kwargs: Unpack[ListNamedShadowsForThingRequestTypeDef]
    ) -> ListNamedShadowsForThingResponseTypeDef:
        """
        Lists the shadows for the specified thing.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/client/list_named_shadows_for_thing.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#list_named_shadows_for_thing)
        """

    def list_retained_messages(
        self, **kwargs: Unpack[ListRetainedMessagesRequestTypeDef]
    ) -> ListRetainedMessagesResponseTypeDef:
        """
        Lists summary information about the retained messages stored for the account.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/client/list_retained_messages.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#list_retained_messages)
        """

    def publish(self, **kwargs: Unpack[PublishRequestTypeDef]) -> EmptyResponseMetadataTypeDef:
        """
        Publishes an MQTT message.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/client/publish.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#publish)
        """

    def update_thing_shadow(
        self, **kwargs: Unpack[UpdateThingShadowRequestTypeDef]
    ) -> UpdateThingShadowResponseTypeDef:
        """
        Updates the shadow for the specified thing.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/client/update_thing_shadow.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#update_thing_shadow)
        """

    def get_paginator(  # type: ignore[override]
        self, operation_name: Literal["list_retained_messages"]
    ) -> ListRetainedMessagesPaginator:
        """
        Create a paginator for an operation.

        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/iot-data/client/get_paginator.html)
        [Show types-boto3 documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/client/#get_paginator)
        """



"""
Type annotations for iot-data service type definitions.

[Documentation](https://youtype.github.io/types_boto3_docs/types_boto3_iot_data/type_defs/)

Copyright 2025 Vlad Emelianov

Usage::

    ```python
    from types_boto3_iot_data.type_defs import BlobTypeDef

    data: BlobTypeDef = ...
    ```
"""

from __future__ import annotations

import sys
from typing import IO, Any, Union

from botocore.response import StreamingBody

from .literals import PayloadFormatIndicatorType

if sys.version_info >= (3, 9):
    from builtins import dict as Dict
    from builtins import list as List
else:
    from typing import Dict, List
if sys.version_info >= (3, 12):
    from typing import NotRequired, TypedDict
else:
    from typing_extensions import NotRequired, TypedDict

__all__ = (
    "BlobTypeDef",
    "DeleteThingShadowRequestTypeDef",
    "DeleteThingShadowResponseTypeDef",
    "EmptyResponseMetadataTypeDef",
    "GetRetainedMessageRequestTypeDef",
    "GetRetainedMessageResponseTypeDef",
    "GetThingShadowRequestTypeDef",
    "GetThingShadowResponseTypeDef",
    "ListNamedShadowsForThingRequestTypeDef",
    "ListNamedShadowsForThingResponseTypeDef",
    "ListRetainedMessagesRequestPaginateTypeDef",
    "ListRetainedMessagesRequestTypeDef",
    "ListRetainedMessagesResponseTypeDef",
    "PaginatorConfigTypeDef",
    "PublishRequestTypeDef",
    "ResponseMetadataTypeDef",
    "RetainedMessageSummaryTypeDef",
    "UpdateThingShadowRequestTypeDef",
    "UpdateThingShadowResponseTypeDef",
)

BlobTypeDef = Union[str, bytes, IO[Any], StreamingBody]

class DeleteThingShadowRequestTypeDef(TypedDict):
    thingName: str
    shadowName: NotRequired[str]

class ResponseMetadataTypeDef(TypedDict):
    RequestId: str
    HTTPStatusCode: int
    HTTPHeaders: Dict[str, str]
    RetryAttempts: int
    HostId: NotRequired[str]

class GetRetainedMessageRequestTypeDef(TypedDict):
    topic: str

class GetThingShadowRequestTypeDef(TypedDict):
    thingName: str
    shadowName: NotRequired[str]

class ListNamedShadowsForThingRequestTypeDef(TypedDict):
    thingName: str
    nextToken: NotRequired[str]
    pageSize: NotRequired[int]

class PaginatorConfigTypeDef(TypedDict):
    MaxItems: NotRequired[int]
    PageSize: NotRequired[int]
    StartingToken: NotRequired[str]

class ListRetainedMessagesRequestTypeDef(TypedDict):
    nextToken: NotRequired[str]
    maxResults: NotRequired[int]

class RetainedMessageSummaryTypeDef(TypedDict):
    topic: NotRequired[str]
    payloadSize: NotRequired[int]
    qos: NotRequired[int]
    lastModifiedTime: NotRequired[int]

class PublishRequestTypeDef(TypedDict):
    topic: str
    qos: NotRequired[int]
    retain: NotRequired[bool]
    payload: NotRequired[BlobTypeDef]
    userProperties: NotRequired[str]
    payloadFormatIndicator: NotRequired[PayloadFormatIndicatorType]
    contentType: NotRequired[str]
    responseTopic: NotRequired[str]
    correlationData: NotRequired[str]
    messageExpiry: NotRequired[int]

class UpdateThingShadowRequestTypeDef(TypedDict):
    thingName: str
    payload: BlobTypeDef
    shadowName: NotRequired[str]

class DeleteThingShadowResponseTypeDef(TypedDict):
    payload: StreamingBody
    ResponseMetadata: ResponseMetadataTypeDef

class EmptyResponseMetadataTypeDef(TypedDict):
    ResponseMetadata: ResponseMetadataTypeDef

class GetRetainedMessageResponseTypeDef(TypedDict):
    topic: str
    payload: bytes
    qos: int
    lastModifiedTime: int
    userProperties: bytes
    ResponseMetadata: ResponseMetadataTypeDef

class GetThingShadowResponseTypeDef(TypedDict):
    payload: StreamingBody
    ResponseMetadata: ResponseMetadataTypeDef

class ListNamedShadowsForThingResponseTypeDef(TypedDict):
    results: List[str]
    timestamp: int
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]

class UpdateThingShadowResponseTypeDef(TypedDict):
    payload: StreamingBody
    ResponseMetadata: ResponseMetadataTypeDef

class ListRetainedMessagesRequestPaginateTypeDef(TypedDict):
    PaginationConfig: NotRequired[PaginatorConfigTypeDef]

class ListRetainedMessagesResponseTypeDef(TypedDict):
    retainedTopics: List[RetainedMessageSummaryTypeDef]
    ResponseMetadata: ResponseMetadataTypeDef
    nextToken: NotRequired[str]
